/* 
 *  Stratégies Numériques
 *  http://www.strat-num.fr/?DR2D-J2S2014
 *  
 */

"use strict";

// First, checks if it isn't implemented yet.
if (!String.prototype.format) {
  String.prototype.format = function() {
    var args = arguments;
    return this.replace(/{(\d+)}/g, function(match, number) { 
      return typeof args[number] !== 'undefined'
        ? args[number]
        : match
      ;
    });
  };
}

(function () {
    "use strict";

    Function.prototype.bindMe = function () {
        var fn = this;
        return function () {
            return fn.apply(DR2D_WSAPI, arguments);
        };
    };
    
   function isFunction(functionToCheck) {
     var getType = {};
     return functionToCheck 
             && getType.toString.call(functionToCheck) === '[object Function]';
   };
   
    /**
     * @namespace DR2D_WSAPI
     * 
     * @property {function}     onWsOpen                - function handler when webSocket is opened
     * @property {function}     onWsMessage             - function handler when webSocket received a message
     * @property {function}     onWsClose               - function handler when webSocket has closed
     * @property {function}     onWsError               - function handler when webSocket has raised an error
     * 
     * @property {object}       host                    - the uri value that describe host
     * @property {function}     connect                 - call this to connect to host
     * @property {function}     disconnect              - call this to disconnect from host
     * 
     * @property {function}     sendSms1                - to send an SMS
     * @property {function}     extractReplyFromCall    - helper to build a reply
     * @property {function}     sendReply               - To send a reply
     */
    var DR2D_WSAPI = 
    {
        _writeToLog: function(o1)
        {
          console.log("DEBUG> "+o1);  
        },

        _ws: null,          //> the WebSocket instance

        host: null,         //> the host uri

        /* the four callbacks to implement */
        onWsOpen: null,     //> the WebSocket has opened
        onWsMessage: null,  //> the WebSocket received a message
        onWsClose: null,    //> the WebSocket has closed
        onWsError: null,    //> the WebSocket has raised an error
        
        connect: function()
        {
          console.assert(!(this._ws),"heu ....");
          if(this._ws!==null) this.disconnect();
          
          this._writeToLog("try to connect to " + this.host);
          this._ws = new WebSocket(this.host);

          this._ws.onopen = function (evt) {
            this._writeToLog("this._ws.onopen");
            if(isFunction(this.onWsOpen)) this.onWsOpen(evt);
          }.bindMe();

          this._ws.onmessage = function (evt) {
            //this._writeToLog("this._ws.onmessage:"+evt.data);
            if(isFunction(this.onWsMessage)) this.onWsMessage(evt);
          }.bindMe();

          this._ws.onclose = function (evt) {
            this._writeToLog("this._ws.onclose[code:"+evt.code+", reason:"+evt.reason+"]");
            if(isFunction(this.onWsClose)) this.onWsClose(evt);
            this._ws=null;
          }.bindMe();

          this._ws.onerror = function (evt) {
            this._writeToLog("this._ws.onerror: "+evt.type);
            if(isFunction(this.onWsError)) this.onWsError(evt);
          }.bindMe();

        }.bindMe(),
        
        disconnect: function()
        {
            this._ws.close();
            //this._ws=null; // done on the onclose event handler
        }.bindMe(),
            
        //> Send a string message
        _sendMsg: function(msg) {
          if(this._ws===null) return;
          this._ws.send(msg);
        }, // pas de '.bindMe()' pour rester en private
             
        //> Send a ShortMessage Service
        sendSms1: function(msg)
        {
            if(!msg) return;
            var sms = {
                ShortMessageService: {
                    msgCode: null,
                    msgDesc: msg,
                    msgArgs:null,
                    msgTs : (new Date()).getTime()
                    }
            };
            var msgJSon = JSON.stringify(sms);
            this._sendMsg(msgJSon);
        }.bindMe(),
        
        extractReplyFromCall: function(stgyCallMsgBulk)
        {
            var msgBulk = {
                stgyReply:{
                    seq: stgyCallMsgBulk.stgyCall.seq,
                    token: stgyCallMsgBulk.stgyCall.token,
                    argsRef: stgyCallMsgBulk.stgyCall.argsRef
                }
            };
            return msgBulk;
        }.bindMe(),
                
        sendReply: function( msgBulk){
            if(!msgBulk) return;
            var msgJSon = JSON.stringify(msgBulk);
            this._sendMsg(msgJSon);
        }.bindMe(),
        
        
        Z: null
    };

    // declare the nameSpace
    window.DR2D_WSAPI = DR2D_WSAPI;

})();       

